#include <stdio.h>
#include <unistd.h>

#include "uart_console.h"

/* Driver configuration */
#include "ti_drivers_config.h"

#include <ti/drivers/GPIO.h>
#include <ti/drivers/SD.h>

#define SUCCESS (0)
#define FAILURE (-1)

SD_Handle sdcard_handle = NULL;

static int init(void)
{
    Board_init();
    UARTConsole_init();
    GPIO_init();
    SD_init();

    /* Power off */
    GPIO_write(CONFIG_GPIO_N_PERIPHERY_ON, 1);
    GPIO_write(CONFIG_GPIO_AUDIO_PWDN, 0);

    usleep(100000);

    /* Power on */
    GPIO_write(CONFIG_GPIO_N_PERIPHERY_ON, 0);
    GPIO_write(CONFIG_GPIO_AUDIO_PWDN, 1);

    usleep(100000);

    sdcard_handle = SD_open(CONFIG_SD_0, NULL);
    if(sdcard_handle == NULL) {
        printf("Error: failed to open the SD card.\n");
        return FAILURE;
    }

    if(SD_initialize(sdcard_handle) != SD_STATUS_SUCCESS) {
        printf("Error: failed to initialize the SD card.\n");
        return FAILURE;
    }

    return SUCCESS;
}

static int sdcard_test(void)
{
    int result;
    uint32_t n_sectors, sector_size;

    n_sectors = SD_getNumSectors(sdcard_handle);
    sector_size = SD_getSectorSize(sdcard_handle);

    printf("n_sectors=%lu, sector_size=%lu\n", n_sectors, sector_size);

    uint8_t buffer[513] = { 0 };

    // Aligned buffer -> OK
    // result = SD_write(sdcard_handle, buffer, 0, 1);

    // Misaligned buffer -> crash
    result = SD_write(sdcard_handle, buffer+1, 0, 1);
    if(result != SD_STATUS_SUCCESS) {
        printf("Failed to write to SD card, error=%d\n", result);
        return FAILURE;
    }

    printf("Write done.\n");

    return SUCCESS;
}

void *mainThread(void *arg0)
{
    if(init() != SUCCESS) {
        printf("Hardware init failed, halting.\n");
        while(1) { sleep(1); }
    }

    printf("Starting SD card test...\n");

    int test_result = sdcard_test();

    if (test_result == SUCCESS) {
        printf("SD test passed.\n");
    } else {
        printf("SD test failed.\n");
    }

    SD_close(sdcard_handle);

    while(1) {
        sleep(1);
    }

    return (NULL);
}
