#include "uart_console.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>

#include <FreeRTOS.h>
#include <semphr.h>

#include <ti/drivers/UART.h>

#include "ti_drivers_config.h"

static UART_Handle console_uart;
static SemaphoreHandle_t uart_sem = NULL;

void UARTConsole_init(void)
{
    UART_init();

    UART_Params params;
    UART_Params_init(&params);
    params.baudRate = 921600;
    params.readMode = UART_MODE_BLOCKING;
    params.readTimeout = UART_WAIT_FOREVER;

    console_uart = UART_open(CONFIG_UART_0, &params);

    if(uart_sem == NULL) {
        uart_sem = xSemaphoreCreateMutex();
        if(uart_sem == NULL) {
            const char msg[] = "Failed to initialize UART mutex, halting.\n";
            UART_write(console_uart, msg, sizeof(msg)-1);
            while(1) {
                sleep(1);
            }
        }
    }
}

int _write(int file, char *ptr, int len)
{
    xSemaphoreTake(uart_sem, portMAX_DELAY);

    if (file == STDOUT_FILENO || file == STDERR_FILENO) {
        UART_write(console_uart, ptr, len);
        xSemaphoreGive(uart_sem);
        return len;
    }
    errno = EIO;
    xSemaphoreGive(uart_sem);
    return -1;
}

int _close(int fd)
{
    // not implemented
    errno = EIO;
    return -1;
}

int _fstat(int fd, struct stat* buf)
{
    // not implemented
    errno = EFAULT;
    return -1;
}

int _isatty(int fd)
{
    if(fd == STDOUT_FILENO || fd == STDERR_FILENO) {
        return 1;
    } else {
        errno = EBADF;
        return 0;
    }
}

off_t _lseek(int fd, off_t offset, int whence)
{
    // not implemented
    errno = EBADF;
    return (off_t) -1;
}

ssize_t _read(int fd, void* buf, size_t count)
{
    // not implemented
    errno = EBADF;
    return -1;
}

